<?php

namespace Litecms\Campaign\Http\Controllers;

use Exception;
use Litepie\Http\Controllers\ResourceController as BaseController;
use Litecms\Campaign\Forms\Campaign as CampaignForm;
use Litecms\Campaign\Http\Requests\CampaignResourceRequest;
use Litecms\Campaign\Http\Resources\CampaignResource;
use Litecms\Campaign\Http\Resources\CampaignsCollection;
use Litecms\Campaign\Models\Campaign;
use Litecms\Campaign\Models\CampaignActions;


/**
 * Resource controller class for campaign.
 */
class CampaignResourceController extends BaseController
{

    /**
     * Initialize campaign resource controller.
     *
     *
     * @return null
     */
    public function __construct()
    {
        parent::__construct();
        $this->form = CampaignForm::grouped(false)
                        ->setAttributes()
                        ->toArray();
        $this->modules = $this->modules(config('litecms.campaign.modules'), 'campaign', guard_url('campaign'));
    }

    /**
     * Display a list of campaign.
     *
     * @return Response
     */
    public function index(CampaignResourceRequest $request)
    {
        $request = $request->all();
        $page = CampaignActions::run('paginate', $request);

        $data = new CampaignsCollection($page);

        $form = $this->form;
        $modules = $this->modules;

        return $this->response->setMetaTitle(trans('campaign::campaign.names'))
            ->view('campaign::campaign.index')
            ->data(compact('data', 'modules', 'form'))
            ->output();

    }

    /**
     * Display campaign.
     *
     * @param Request $request
     * @param Model   $campaign
     *
     * @return Response
     */
    public function show(CampaignResourceRequest $request, Campaign $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CampaignResource($model);
        return $this->response
            ->setMetaTitle(trans('app.view') . ' ' . trans('campaign::campaign.name'))
            ->data(compact('data', 'form', 'modules'))
            ->view('campaign::campaign.show')
            ->output();
    }

    /**
     * Show the form for creating a new campaign.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function create(CampaignResourceRequest $request, Campaign $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CampaignResource($model);
        return $this->response->setMetaTitle(trans('app.new') . ' ' . trans('campaign::campaign.name'))
            ->view('campaign::campaign.create')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Create new campaign.
     *
     * @param Request $request
     *
     * @return Response
     */
    public function store(CampaignResourceRequest $request, Campaign $model)
    {
        try {
            $request = $request->all();
            $model = CampaignAction::run('store', $model, $request);
            $data = new CampaignResource($model);
            return $this->response->message(trans('messages.success.created', ['Module' => trans('campaign::campaign.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('campaign/campaign/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('/campaign/campaign'))
                ->redirect();
        }

    }

    /**
     * Show campaign for editing.
     *
     * @param Request $request
     * @param Model   $campaign
     *
     * @return Response
     */
    public function edit(CampaignResourceRequest $request, Campaign $model)
    {
        $form = $this->form;
        $modules = $this->modules;
        $data = new CampaignResource($model);
        // return view('campaign::campaign.edit', compact('data', 'form', 'modules'));

        return $this->response->setMetaTitle(trans('app.edit') . ' ' . trans('campaign::campaign.name'))
            ->view('campaign::campaign.edit')
            ->data(compact('data', 'form', 'modules'))
            ->output();

    }

    /**
     * Update the campaign.
     *
     * @param Request $request
     * @param Model   $campaign
     *
     * @return Response
     */
    public function update(CampaignResourceRequest $request, Campaign $model)
    {
        try {
            $request = $request->all();
            $model = CampaignAction::run('update', $model, $request);
            $data = new CampaignResource($model);

            return $this->response->message(trans('messages.success.updated', ['Module' => trans('campaign::campaign.name')]))
                ->code(204)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('campaign/campaign/' . $model->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('campaign/campaign/' .  $model->getRouteKey()))
                ->redirect();
        }

    }

    /**
     * Remove the campaign.
     *
     * @param Model   $campaign
     *
     * @return Response
     */
    public function destroy(CampaignResourceRequest $request, Campaign $model)
    {
        try {

            $request = $request->all();
            $data = new CampaignResource($model);

            return $this->response->message(trans('messages.success.deleted', ['Module' => trans('campaign::campaign.name')]))
                ->code(202)
                ->status('success')
                ->data(compact('data'))
                ->url(guard_url('campaign/campaign/0'))
                ->redirect();

        } catch (Exception $e) {

            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('campaign/campaign/' .  $model->getRouteKey()))
                ->redirect();
        }

    }
}
